-- composants transmetteur serie
-- comp_tx.vhd

entity controle is
   port ( hor, reset, plein : in bit ;
          compte : in integer range 0 to 7 ;
          razplein, shi_ld, razreg, razcpt : out bit ) ;
end entity controle ;

architecture fsm of controle is
   subtype etat4 is bit_vector(1 downto 0) ;
   constant init   : etat4 := "11" ;
   constant load   : etat4 := "01" ;
   constant shift1 : etat4 := "10" ;
   constant shift  : etat4 := "00" ;
   signal etat : etat4 ; -- machine d'tats contrleur
begin

razcpt   <= '1' when etat = init or etat =  load else '0' ;
razplein <= '1' when etat = load else '0' ;
razreg   <= '1' when etat = init else '0' ;
shi_ld   <= '0' when etat = load else '1' ;

controleur : process
   begin
   wait until hor = '1' ;
   if reset = '1' then
      etat <= init ;
   else
      case etat is
         when init   => if plein = '1' then 
                           etat <= load ; 
                        end if ;
         when load   => etat <= shift1 ;
         when shift1 => etat <= shift  ; 
         when shift  => if compte = 7 then
                           etat <= init ; 
                        end if ;
      end case ;
   end if ;
   end process controleur ;
end fsm ;

entity dialogue is
   port ( hor, ds, reset, raz : in bit ;
          bfull : buffer bit ; 
          plein : out bit ) ;
end entity dialogue ;

architecture saxo of dialogue is
begin
rs_bfull : process ( ds, hor, reset)
   begin
      if reset = '1' then
         bfull <= '0' ; 
      elsif ds = '0' then
         bfull <= '1' ;
      elsif hor'event and hor = '1' and raz = '1' then
         bfull <= '0' ;
      end if ;
end process rs_bfull ;

buf_plein : process
   begin
      wait until hor = '1' ;
      if raz = '1' then
         plein <= '0' ;
      elsif  ds ='1' then
         plein <= bfull ;
      end if ;
end process buf_plein ;
end saxo ;


entity latch is 
   port ( ds   : in bit ;
          din  : in bit_vector (7 downto 0) ;
          dout : out bit_vector (7 downto 0) ) ;
end entity latch ;

architecture comporte of latch is
begin
buf_in : process (ds, din)
   begin
      if ds = '0' then
         dout <= din ;
      end if ;
end process buf_in ;
end comporte ;

entity compteur is 
   generic ( taille : integer := 8 ) ;
   port ( hor, raz : in bit ;
          compte : buffer integer range 0 to taille - 1 ) ;
end compteur ;

architecture simple of compteur is
begin
cpt : process
   begin
      wait until hor = '1' ;
      if raz = '1' then
         compte <= 0 ;
      else
         compte <= (compte + 1) mod taille ;
      end if ;
end process cpt ;
end simple ;

entity par_serie is
   port ( hor, raz, shi_ld : in bit ;
          din : in bit_vector (8 downto 0) ;
          sout : out bit ) ;
end entity par_serie ;

architecture boucle of par_serie is
signal buf : bit_vector (8 downto 0) ;
begin
sout <= buf(0) ;
regem : process
   begin
      wait until hor = '1' ;
      if raz = '1' then
         buf <= (others => '0') ;
      elsif shi_ld = '0' then
         buf <= din ;
      else
         buf(8) <= '0' ;
         for i in 0 to 7 loop
            buf(i) <= buf(i + 1) ;
         end loop ;
      end if ;
end process regem ;
end boucle ;
