-- transmetteur srie, une architecture, plusieurs process, synthse warp
-- transmi3.vhd

entity ser_tx is
   port ( hor, ds, reset : in bit ;
          din  : in bit_vector(7 downto 0) ;
          bfull: buffer bit ;
          sout : out bit ) ;
end ser_tx ;

architecture multi_proc of ser_tx is
   subtype etat is bit_vector(1 downto 0) ;
   constant init   : etat := "11" ;
   constant load   : etat := "01" ;
   constant shift1 : etat := "10" ;
   constant shift  : etat := "00" ;
   signal controle : etat ; -- machine d'tats contrleur
   signal compte   : integer range 0 to 7 ; -- 8 bits  mettre
   signal bufin : bit_vector(7 downto 0) ; -- buffer latch entre
   signal bufem : bit_vector(8 downto 0) ; -- registre  dcalage
   signal razcpt : bit ; -- raz compteur mission
   signal shi_ld : bit ; -- commande dcalage
   signal razreg : bit ; -- raz dcalage
   signal razplein : bit ; -- mise  '0' bascule plein et bfull
   signal plein  : bit ; -- bascule interne,synchrone buffer full
begin

sout     <= bufem(0) ;
razcpt   <= '1' when controle = init or controle =  load else '0' ;
razplein <= '1' when controle = load else '0' ;
razreg   <= '1' when controle = init else '0' ;
shi_ld   <= '0' when controle = load else '1' ;

buf_plein : process
   begin
      wait until hor = '1' ;
      if razplein = '1' then
         plein <= '0' ;
      elsif  ds ='1' then
         plein <= bfull ;
      end if ;
end process buf_plein ;

rs_bfull : process ( ds, hor, reset)
   begin
      if reset = '1' then
         bfull <= '0' ;
      elsif ds = '0' then
         bfull <= '1' ;
      elsif hor'event and hor = '1' then
         if razplein = '1' then
            bfull <= '0' ;
         end if ;
      end if ;
end process rs_bfull ;

buf_in : process (ds, din)
   begin
      if ds = '0' then
         bufin <= din ;
      end if ;
end process buf_in ;

compteur : process
   begin
      wait until hor = '1' ;
      if razcpt = '1' then
         compte <= 0 ;
      else
         compte <= (compte + 1) mod 8 ;
      end if ;
end process compteur ;

regem : process
   begin
      wait until hor = '1' ;
      if razreg = '1' then
         bufem <= (others => '0') ;
      elsif shi_ld = '0' then
         bufem(8 downto 1) <= bufin ;
         bufem(0) <= '1' ;
      else
         bufem(8) <= '0' ;
         for i in 0 to 7 loop
            bufem(i) <= bufem(i + 1) ;
         end loop ;
      end if ;
end process regem ;

controleur : process
   begin
   wait until hor = '1' ;
   if reset = '1' then
      controle <= init ;
   else
      case controle is
         when init   => if plein = '1' then 
                           controle <= load ; 
                        end if ;
         when load   => controle <= shift1 ;
         when shift1 => controle <= shift  ; 
         when shift  => if compte = 7 then
                           controle <= init ; 
                        end if ;
      end case ;
   end if ;
end process controleur ;

end multi_proc ;
